@@ -13,6 +13,7 @@
 #include "quickdialog.h"
 #include "screencast.h"
 #include <QEventLoop>
+#include <QRect>
 
 class ScreenChooserDialog : public QuickDialog
 {
@@ -22,11 +23,20 @@ public:
     ~ScreenChooserDialog() override;
 
     QList<Output> selectedOutputs() const;
-    QVector<QMap<int, QVariant>> selectedWindows() const;
+    QList<QMap<int, QVariant>> selectedWindows() const;
     bool allowRestore() const;
+    QRect selectedRegion() const;
+
+public Q_SLOTS:
+    void accept() override;
 
 Q_SIGNALS:
     void clearSelection();
+
+private:
+    void setRegion(const QRect region);
+
+    QRect m_region;
 };
 
 #endif // XDG_DESKTOP_PORTAL_KDE_SCREENCHOOSER_DIALOG_H
